//*********************************************************
// www.Ultimategameprogramming.com
// Cel-Shading shader demo.
// Created by the Programming Ace. Modified by Raphael Mun
//
// Uniform Variables:
//   worldViewProj   - Holds the model view projection matrix.
//   worldView       - Holds the model view matrix.
//   lightPos        - Holds object space light position.
//   cameraPos       - Holds camera position.
//   shininess       - Holds specular power value.
//   diffuseTex      - Holds 1D diffuse ramp texture.
//   specularTex     - Holds 1D specular ramp texture.
//   edgeTex         - Holds 1D edge ramp texture.
//   matDiffuse      - Holds diffuse material.
//   matSpecular     - Holds specular material.
//
//*********************************************************


// Uniform parameters.
float4x4 worldViewProj  : WorldViewProjection;
float4x4 worldView      : WorldView;

float3 lightPos;
float3 cameraPos;
float shininess;

texture Texture0; // This texture will be loaded by the application
texture Texture1;
texture Texture2;
texture Texture3;

texture diffuseTex;
texture specularTex;

float4 matDiffuse;
float4 matSpecular;


sampler DiffuseSampler = sampler_state
{
    Texture   = (diffuseTex);
    MipFilter = POINT;
    MinFilter = POINT;
    MagFilter = POINT;
};

sampler SpecularSampler = sampler_state
{
    Texture   = (specularTex);
    MipFilter = POINT;
    MinFilter = POINT;
    MagFilter = POINT;
};

// Vertex shader input.
struct VS_Input
{
   float3 vertexPos  : POSITION;
   float3 normal     : NORMAL;
};


// Vertex shader output.
struct Vs_Output
{
   float4 vertexPos  : POSITION;
   float3 color      : COLOR0;
   float diffuse     : TEXCOORD0;
   float specular    : TEXCOORD1;
};


// Pixel shader output.
struct Ps_Output
{
	float4 color      : COLOR;
};


// The vertex shader...
Vs_Output VertexShaderEffect(VS_Input IN)
{
   Vs_Output vs_out;

   // Set output vertex pos, and texture coords.
   vs_out.vertexPos = mul(worldViewProj, float4(IN.vertexPos, 1));
   vs_out.color = float3(1.0f, 1.0f, 1.0f);
   
   // Calculate light vector.
   float4 pos = mul(worldView, IN.vertexPos);
   float3 lightVec = normalize(lightPos - pos);

   // Renormalize the normal.
   float3 n = mul((float3x3)worldView, IN.normal);
   
   // Calculate the diffuse term (n dot l).
   vs_out.diffuse = max(dot(n, lightVec), 0);

   // Get the eye vector and half angle.
   float3 eyeVec = normalize(cameraPos - pos);
   float3 halfAngle = normalize(lightVec + eyeVec);
   
   // Calculate specular term (pow(max(n dot h), 0) shinines).
   vs_out.specular = pow(max(dot(n, halfAngle), 0), shininess);

   // if no diffuse, then no specular.
   if(vs_out.diffuse <= 0) vs_out.specular = 0;
   
   return vs_out;
}


// The pixel shader...
Ps_Output PixelShaderEffect(Vs_Output IN)
{
   Ps_Output ps_out;
   
   float diffuseCol = tex1D(DiffuseSampler, IN.diffuse).x;
   float specularCol = tex1D(SpecularSampler, IN.specular).x;
   
   ps_out.color = float4(IN.color * ((matDiffuse * diffuseCol +
                         matSpecular * specularCol)), 1);

   return ps_out;
}

// Outline Generator
Vs_Output VertexShaderOutline(VS_Input IN)
{
   Vs_Output vs_out;

   // Set output vertex pos, and texture coords.
   vs_out.vertexPos = mul(worldViewProj, float4(IN.vertexPos, 1));
   vs_out.color = float3(0.0f, 0.0f, 0.0f);
   
   // Calculate the diffuse term (n dot l).
   vs_out.diffuse = 0;//max(dot(n, lightVec), 0);
   
   // Calculate specular term (pow(max(n dot h), 0) shininess).
   vs_out.specular = 0;//pow(max(dot(n, halfAngle), 0), shininess);

   // if no diffuse, then no specular.
   if(vs_out.diffuse <= 0) vs_out.specular = 0;
   
   return vs_out;
}


// Effect...
technique CelShading
{
   pass Pass0
   {
      // Here we will compile and set the vertex and pixel shader.
      VertexShader = compile vs_2_0 VertexShaderEffect();
      PixelShader  = compile ps_2_0 PixelShaderEffect();
   }
   pass Pass1
   {
      CullMode = CW;
      FillMode = Wireframe;
      DepthBias = 1;
      VertexShader = compile vs_2_0 VertexShaderOutline();
   }
}